@@ -18,7 +18,10 @@
 #' }
 #' where \eqn{Q} denotes the number of models included in the averaging procedure.
 #' @references Schorning K, Bornkamp B, Bretz F, Dette H. 2016. Model selection versus model averaging in dose finding studies. Stat Med; 35; 4021-4040.
-#' @param models List (or vector) of model names for which a fit will be performed.
+#' @param models A Mods object as created with `DoseFinding::Mods()` or a vector
+#' of model names for which a fit will be performed.
+#' Implemented model shapes are `"linear"`, `"exponential"`, `"logistic"`,
+#' `"emax"`, `"sigEmax"`, `"quadratic"`, and `"betaMod"`.
 #' @param dose_levels A vector containing the different dosage levels.
 #' @param posterior A getPosterior object, containing the (multivariate) posterior distribution per dosage level.
 #' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
@@ -58,19 +61,31 @@ getModelFits <- function (
 
 ) {
   
+  checkmate::assert(
+    checkmate::check_class(models, "Mods"),
+    checkmate::check_character(models, any.missing = FALSE)
+  )
+  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE)
+  checkmate::assert(
+    checkmate::check_class(posterior, "postList"),
+    checkmate::check_list(posterior) && all(sapply(posterior, inherits, c("normMix", "mix")))
+  )
+  checkmate::assert_logical(avg_fit)
+  checkmate::assert_logical(simple)
+  
   if (inherits(models, "character")) {
-    models <- stats::setNames(as.list(models), models)
+    
+    model_names <- models
+    
+  } else if (inherits(models, "Mods")) {
+    
+    model_names <- names(models)
+    
   }
   
-  checkmate::check_list(models, any.missing = FALSE)
-  checkmate::check_double(dose_levels, lower = 0, any.missing = FALSE, len = length(models))
-  checkmate::check_class(posterior, "postList")
-  checkmate::check_logical(avg_fit)
-  checkmate::check_logical(simple)
+  model_names <- sort(unique(gsub("\\d", "", model_names)))
 
-  model_names <- unique(gsub("\\d", "", names(models)))
-
-  getModelFit <- ifelse(simple, getModelFitSimple, getModelFitOpt)
+  getModelFit <- if (simple) getModelFitSimple else getModelFitOpt
 
   model_fits  <- lapply(model_names, getModelFit, dose_levels, posterior, list("scal" = attr(models, "scal")))
   model_fits  <- addModelWeights(model_fits)
@@ -83,6 +98,7 @@ getModelFits <- function (
     
   }
 
+  attr(model_fits, "direction") <- getDirection(models, model_fits)
   attr(model_fits, "posterior") <- posterior
   class(model_fits)             <- "modelFits"
 
@@ -90,6 +106,49 @@ getModelFits <- function (
 
 }
 
+getDirection <- function (
+    
+  models,
+  model_fits
+  
+) {
+
+  if (inherits(models, "Mods")) {
+    
+    direction   <- attr(models, "direction")
+    
+  } else if (inherits(models, "character")) {
+    
+    ## Try guessing the direction from the fitted model
+    ## in case a character vector is provided
+    
+    preds <- model_fits[[1]]$pred_values
+    
+    if (min(preds[-1]) < preds[1]) {
+      ## min non-placebo effect less than placebo
+      
+      direction <- "decreasing"
+      
+    } else if (max(preds[-1]) > preds[1]) {
+      ## max non-placebo effect greater than placebo
+      
+      direction <- "increasing"
+      
+    } else {
+      
+      direction <- character(0)
+      
+      warning(paste0("The direction of the beneficial direction ",
+                     "with increasing dose levels could not be determined."))
+      
+    }
+    
+  }
+  
+  return (direction)
+  
+}
+
 addAvgFit <- function (
     
   model_fits,
@@ -107,7 +166,7 @@ addAvgFit <- function (
                    gAIC         = NA,
                    model_weight = NA)
   
-  model_fits <- c(model_fits, avgFit = list(avg_fit))
+  model_fits <- c(avgFit = list(avg_fit), model_fits)
   
   return (model_fits)
   
@@ -272,18 +331,17 @@ getModelFitOpt <- function (
   }
 
   param_list <- getOptParams(model, dose_levels, posterior)
-  post_combs <- getPostCombsI(posterior)
+  post_combs <- getPostCompsI(posterior)
 
   fit <- nloptr::nloptr(
-    eval_f = optFun,
-    x0     = param_list$params$x0,
-    lb     = param_list$params$lb,
-    ub     = param_list$params$ub,
-    opts   = param_list$opts,
+    eval_f      = optFun,
+    x0          = param_list$params$x0,
+    lb          = param_list$params$lb,
+    ub          = param_list$params$ub,
+    opts        = param_list$opts,
     dose_levels = dose_levels,
     post_combs  = post_combs,
-    expr_i      = param_list$expr_i
-  )
+    expr_i      = param_list$expr_i)
 
   names(fit$solution) <- param_list$c_names
 
@@ -434,7 +492,7 @@ addModelWeights <- function (
 #'                                
 #' # MED based on bootstrapped quantiles
 #' bs_quantiles <- getBootstrapQuantiles(model_fits = model_fits,
-#'                                       quantiles  = c(0.025, 0.2, 0.5),
+#'                                       quantiles  = c(0.025, 0.2, 0.5, 0.8),
 #'                                       n_samples  = 100) # speeding up example run time
 #'                                       
 #' getMED(delta          = 5,
@@ -452,17 +510,22 @@ getMED <- function (
   
 ) {
   
+  ## R CMD --as-cran appeasement
+  model <- sample_type <- NULL
+  
   stopifnot("Either model_fits or bs_quantiles must be not NULL, but not both" =
               is.null(model_fits) & !is.null(bs_quantiles) |
               !is.null(model_fits) & is.null(bs_quantiles))
   
-  checkmate::check_double(delta)
-  checkmate::check_double(evidence_level, lower = 0, upper = 1)
-  checkmate::check_double(dose_levels, lower = 0, any.missing = FALSE)
+  checkmate::assert_double(delta)
+  checkmate::assert_double(evidence_level, lower = 0, upper = 1)
+  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE, null.ok = TRUE)
   
   if (!is.null(model_fits)) {
+    ## Direction not needed, because mean value
+    ## (approx. evidence_level = 0.5) is used.
     
-    checkmate::check_class(model_fits, "modelFits")
+    checkmate::assert_class(model_fits, "modelFits")
     
     if (is.null(dose_levels)) {
       
@@ -472,42 +535,50 @@ getMED <- function (
     
     preds <- do.call(rbind, predict.modelFits(model_fits, doses = dose_levels))
     
+    abs_diffs <- abs(preds - preds[, 1])
+    colnames(abs_diffs) <- dose_levels
+    
   } else {
     
+    if (attr(bs_quantiles, "direction") == "decreasing") {
+      
+      bs_quantiles <- bs_quantiles |>
+        dplyr::mutate(q_prob = 1 - q_prob)
+      
+    }
+    
     if (is.null(dose_levels)) {
       
-      dose_levels <- unique(bs_quantiles$doses)
+      dose_levels <- unique(bs_quantiles$dose)
       
     }
     
     # R CMD Check Appeasement
-    q_probs <- doses <- q_values <- models <- NULL
+    q_prob <- dose <- q_val <- models <- NULL
+    
+    # Floating-point precision handling to pick up correct rows of bs_quantiles
+    tolerance <- 1e-9
     
     stopifnot("corresponding quantile (i.e. 1 - evidence_level) not in bootstrapped quantiles matrix" = 
-                evidence_level %in% (1 - bs_quantiles$q_probs))
+                any(abs((1 - bs_quantiles$q_prob) - evidence_level) < tolerance))
     
-    stopifnot("dose_levels not in bootstrapped quantiles matrix" = 
-                all(dose_levels %in% bs_quantiles$doses))
+    stopifnot("dose_levels not (all) in bootstrapped quantiles matrix" = 
+                all(dose_levels %in% bs_quantiles$dose))
     
-    preds <- bs_quantiles |>
-      dplyr::filter((1 - q_probs) %in% evidence_level) |>
-      dplyr::filter(doses %in% dose_levels) |>
-      tidyr::pivot_wider(names_from = doses, values_from = q_values)
-    
-    model_names <- preds$models
-    
-    preds <- preds |>
-      dplyr::select(-models, -q_probs) |>
-      as.matrix()
-    
-    rownames(preds) <- model_names
+    abs_diffs <- bs_quantiles |>
+      dplyr::filter(abs((1 - q_prob) - evidence_level) < tolerance,
+                    dose %in% dose_levels,
+                    sample_type == "diff") |>
+      tidyr::pivot_wider(names_from = dose, values_from = q_val) |>
+      dplyr::select(-model, -sample_type, -q_prob) |>
+      as.matrix() |>
+      abs()
+
+    rownames(abs_diffs) <- unique(bs_quantiles$model)
     
   }
   
-  med_info <- apply(preds, 1, function (model_preds) {
-    
-    # assumes 1st DG to be control DG
-    med_indx_m <- abs(model_preds[2:length(model_preds)] - model_preds[1]) > delta
+  med_info <- apply(abs_diffs > delta, 1, function (med_indx_m) {
     
     if (!any(med_indx_m)) {
       
@@ -517,7 +588,7 @@ getMED <- function (
     } else {
       
       med_reached <- 1
-      med_indx    <- min(which(med_indx_m)) + 1
+      med_indx    <- min(which(med_indx_m))
       med         <- dose_levels[med_indx]
       
     }
