@@ -33,7 +33,7 @@ test_that("getPosterior works correctly", {
   posterior_list <- getPosterior(
     prior_list = prior_list_matrix,
     mu_hat = mu_hat,
-    S_hat = se_hat_vector,
+    S_hat = diag(se_hat_vector),
     calc_ess = FALSE
   )
   expect_type(posterior_list, "list")
@@ -81,20 +81,21 @@ test_that("getPriorList input parameters do work as intented", {
 })
 
 test_that("getPosteriorI works correctly", {
-  # Prepare test data and parameters
-  data_i <- data.frame(
-    dose = c(0, 1, 2, 3, 4),
-    response = c(10, 20, 30, 40, 50)
-  )
 
   #prior_list <- list(1, 2, 3, 4)
   #mu_hat <- c(10, 20, 30, 40)
   #se_hat <- matrix(c(1, 2, 3, 4), nrow = 4, ncol = 1)
 
   # Test getPosteriorI function
-  post_list <- getPosteriorI(data_i = data_i, prior_list = prior_list_matrix, mu_hat = mu_hat, se_hat = se_hat_vector)
+  post_list <- getPosteriorI(data_i = NULL, prior_list = prior_list_matrix, mu_hat = mu_hat, se_hat = se_hat_vector)
   expect_type(post_list, "list")
   expect_s3_class(post_list, "postList")
+  
+  # Prepare test data and parameters
+  data_i <- data.frame(
+    dose = c(0, 1, 2, 3, 4),
+    response = c(10, 20, 30, 40, 50)
+  )
 
   # Test mu_hat and sd_hat both null branch
   post_list <- getPosteriorI(data_i, prior_list_matrix, NULL, NULL)
@@ -117,12 +118,12 @@ test_that("summary.postList works correctly", {
   expect_type(summary_tab, "character")
 })
 
-test_that("getPostCombsI returns an object with correct attributes", {
+test_that("getPostCompsI returns an object with correct attributes", {
   posterior_i <- list(
     matrix(c(2, 1, 2), nrow = 3),
     matrix(c(2, 1, 2), nrow = 3)
   )
-  result <- getPostCombsI(posterior_i)
+  result <- getPostCompsI(posterior_i)
 
   expect_true(is.list(result))
   expect_equal(length(result), 3)
@@ -153,11 +154,13 @@ test_that("postMix2posteriorList works correctly", {
     mu_hat   = mu_hat,
     S_hat    = se_hat_matrix)
 
+  ## TODO: obsolete, as no option for vector is present any more?
+  ## or re-purpose & rename test to check if both implementations yiel the same results?
   # create posterior with vector
   posterior_vector <- getPosterior(
     prior_list = prior_list_matrix,
     mu_hat = mu_hat,
-    S_hat = se_hat_vector,
+    S_hat = diag(se_hat_vector^2),
     calc_ess = FALSE
   )
 
@@ -202,7 +205,7 @@ test_that("postMix2posteriorList works correctly", {
 
   ### test similarity of results for posterior from a matrix compared to posterior from a vector with square roots
   se_hat <- c(1, 2, 3, 4, 5)
-  S_hat  <- diag(se_hat)
+  S_hat  <- diag(se_hat^2)
 
   posterior_matrix_S <- getPosterior(
     prior_list = prior_list_matrix,
@@ -211,18 +214,18 @@ test_that("postMix2posteriorList works correctly", {
     calc_ess   = FALSE
   )
 
-  posterior_vector_se <- getPosterior(
+  posterior_vector_se <- getPosterior_vec(
     prior_list = prior_list_matrix,
     mu_hat     = mu_hat,
-    S_hat      = sqrt(se_hat),
+    S_hat      = se_hat,
     calc_ess   = FALSE
   )
 
-  expect_equal(posterior_matrix_S$Ctr, posterior_vector_se$Ctr)
-  expect_equal(posterior_matrix_S$DG_1, posterior_vector_se$DG_1)
-  expect_equal(posterior_matrix_S$DG_2, posterior_vector_se$DG_2)
-  expect_equal(posterior_matrix_S$DG_3, posterior_vector_se$DG_3)
-  expect_equal(posterior_matrix_S$DG_4, posterior_vector_se$DG_4)
+  expect_equal(posterior_matrix_S$Ctr, posterior_vector_se$Ctr, tolerance = 1e-3)
+  expect_equal(posterior_matrix_S$DG_1, posterior_vector_se$DG_1, tolerance = 1e-3)
+  expect_equal(posterior_matrix_S$DG_2, posterior_vector_se$DG_2, tolerance = 1e-3)
+  expect_equal(posterior_matrix_S$DG_3, posterior_vector_se$DG_3, tolerance = 1e-3)
+  expect_equal(posterior_matrix_S$DG_4, posterior_vector_se$DG_4, tolerance = 1e-3)
 
 
 
